/****************************************************************************
** Meta object code from reading C++ file 'postfetchjob.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../libkgapi-26.07.80/src/blogger/postfetchjob.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'postfetchjob.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t {};
} // unnamed namespace

template <> constexpr inline auto KGAPI2::Blogger::PostFetchJob::qt_create_metaobjectdata<qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KGAPI2::Blogger::PostFetchJob",
        "fetchBodies",
        "fetchImages",
        "maxResults",
        "filterLabels",
        "startDate",
        "endDate",
        "statusFilter",
        "StatusFilters",
        "StatusFilter",
        "Draft",
        "Live",
        "Scheduled",
        "All"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fetchBodies'
        QtMocHelpers::PropertyData<bool>(1, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'fetchImages'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'maxResults'
        QtMocHelpers::PropertyData<uint>(3, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'filterLabels'
        QtMocHelpers::PropertyData<QStringList>(4, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'startDate'
        QtMocHelpers::PropertyData<QDateTime>(5, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'endDate'
        QtMocHelpers::PropertyData<QDateTime>(6, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'statusFilter'
        QtMocHelpers::PropertyData<StatusFilters>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'StatusFilter'
        QtMocHelpers::EnumData<enum StatusFilter>(9, 9, QMC::EnumFlags{}).add({
            {   10, StatusFilter::Draft },
            {   11, StatusFilter::Live },
            {   12, StatusFilter::Scheduled },
            {   13, StatusFilter::All },
        }),
    };
    return QtMocHelpers::metaObjectData<PostFetchJob, qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KGAPI2::Blogger::PostFetchJob::staticMetaObject = { {
    QMetaObject::SuperData::link<KGAPI2::FetchJob::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t>.metaTypes,
    nullptr
} };

void KGAPI2::Blogger::PostFetchJob::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PostFetchJob *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->fetchBodies(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->fetchImages(); break;
        case 2: *reinterpret_cast<uint*>(_v) = _t->maxResults(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->filterLabels(); break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->startDate(); break;
        case 5: *reinterpret_cast<QDateTime*>(_v) = _t->endDate(); break;
        case 6: *reinterpret_cast<StatusFilters*>(_v) = _t->statusFilter(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFetchBodies(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setFetchImages(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setMaxResults(*reinterpret_cast<uint*>(_v)); break;
        case 3: _t->setFilterLabels(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setStartDate(*reinterpret_cast<QDateTime*>(_v)); break;
        case 5: _t->setEndDate(*reinterpret_cast<QDateTime*>(_v)); break;
        case 6: _t->setStatusFilter(*reinterpret_cast<StatusFilters*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KGAPI2::Blogger::PostFetchJob::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KGAPI2::Blogger::PostFetchJob::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6KGAPI27Blogger12PostFetchJobE_t>.strings))
        return static_cast<void*>(this);
    return KGAPI2::FetchJob::qt_metacast(_clname);
}

int KGAPI2::Blogger::PostFetchJob::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KGAPI2::FetchJob::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
